% y = dexp(x) Exponential with denormals truncated
%
% Computes y = exp(x) but returns 0 where y^2 would be close to a denormal.
% Denormals can slow down computations considerably, so if the extra accuracy
% is not really necessary it's better to avoid them. Denormals often appear
% when applying Gaussian kernels to vector pairs that are widely separated,
% and these kernel values are then multiplied times other vectors or other
% kernel values.

% Copyright (c) 2011 by Miguel A. Carreira-Perpinan

function y = dexp(x)

denth = log(10*sqrt(realmin));		% Threshold to prevent denormals

if issparse(x)
  [m,n] = size(x);
  [i,j] = find(x>denth); y = sparse(i,j,exp(x(sub2ind([m n],i,j))),m,n);
else
  z = find(x>denth); y = zeros(size(x)); y(z) = exp(x(z));
end

