% [f,fX,E] = linftrain2(f,X,iXX,Y) Train linf (auxiliary function 2)
%
% See linftrain1.
%
% In:
%   f,X,iXX: see linftrain1.
%   Y: NxD matrix, N D-dim data points rowwise.
% Out:
%   f: (struct) the linear function, with fields:
%      type='linf', weights W (DxL), biases w (Dx1), regularisation
%      parameter l.
%   fX: NxD matrix, f(X).
%   E: 1x2 list, fit error and regularisation error.

% Copyright (c) 2011 by Miguel A. Carreira-Perpinan

function [f,fX,E] = linftrain2(f,X,iXX,Y)

Ym = bsxfun(@minus,Y,mean(Y,1))';
f.W = Ym*iXX; WX = f.W*X'; f.w = mean(Y'-WX,2);

if nargout>1
  fX = bsxfun(@plus,WX,f.w)';	% fX = linf(X,f)
end
if nargout>2
  E = sum(sum((Y-fX).^2));					% Fit error
  if f.l~=0 E = [E f.l*(f.W(:)'*f.W(:))]; else E=[E 0]; end	% Regul. error
end

