% [f,fX,E] = rbftrain2(f,G,iGG,Y) Train RBF (auxiliary function 2)
%
% See rbftrain1.
%
% In:
%   f,G,iGG: see rbftrain1.
%   Y: NxD matrix, N D-dim data points rowwise.
% Out:
%   f: (struct) the RBF, with fields:
%      type='rbf', centres C (MxL), width s, weights W (DxM), biases w (Dx1),
%      regularisation parameter l.
%   fX: NxD matrix, f(X).
%   E: 1x2 list, fit error and regularisation error.

% Copyright (c) 2011 by Miguel A. Carreira-Perpinan

function [f,fX,E] = rbftrain2(f,G,iGG,Y)

Ym = bsxfun(@minus,Y,mean(Y,1))';
f.W = Ym*iGG; WG = f.W*G; f.w = mean(Y'-WG,2);

if nargout>1
  fX = bsxfun(@plus,WG,f.w)';	% fX = rbf(X,f)
end
if nargout>2
  E = sum(sum((Y-fX).^2));	% Fit error
  E = [E f.l*(f.W(:)'*f.W(:))];	% Regul. error
end

