% This script trains an low dimensional SVM on a 3-spirals data set, using
% the method of auxiliary coordinates, and then plots the decision
% boundaries of the final classifier.
% It requires the spirals.mat (provided in the package).

% % The code below was used to generate the data set, including input, SVM
% % labels for one-versus-rest scheme, and class labels.
% N=200;  a=pi/6;  b=4*pi;  t=((b-a)*(linspace(0,1,N).^0.65)+a)';
% K=3;  Y=-ones(N*K,K);
% X=[]; XP={};
% for j=0:K-1
%   tmp=[t.*sin(t+2*pi*j/K) t.*cos(t+2*pi*j/K)];
%   tmp=tmp+randn(size(tmp))/5;
%   X=[X; tmp];  XP{j+1}=tmp;
%   Y(j*N+1:(j+1)*N,j+1)=1;
% end
% [~,label]=max(Y,[],2);
% save spirals K XP X Y label

addpath ./auxi
addpath ./libsvm-3.18/matlab/

load spirals;  Ntrain=size(X,1);
figure(1);  plot(X(:,1),X(:,2),'bo');  set(gca,'DataAspectRatio',[1 1 1]); 
axis([-15 15 -15 15]);  title('data');

% Use 2D latent coordinates and random initialization.
L=K-1;  rng('default');  Zinit=randn(Ntrain,L);

% Set hyper-parameters.
Fmap={'rbf',X,0.4,1e-2};  gmap={'svm',100};
H=drsvm(X,Y,Zinit,Fmap,gmap,[],[],20);

% Compute classification error.
pred=rbf(X,H);  [~,pred]=max(pred,[],2);
fprintf('training error rate: %d/%d\n', sum(pred~=label),length(label));

figure(999);  clf;  hold on;  ax=[-15 15 -15 15];
COLOR={'bo','ro','go','mo','co','ko'};
% Plot data.
for i=1:K
  plot(XP{i}(:,1),XP{i}(:,2),COLOR{i});
end
x=linspace(ax(1),ax(2),50);  [XX,YY]=meshgrid(x);  y=rbf([XX(:),YY(:)],H);
% Plot decision boundaries.
for j=1:K
  contour(XX,YY,reshape(y(:,j),size(XX)),[0 0],'k','linewidth',4);
end
set(gca,'DataAspectRatio',[1 1 1]);  axis(ax);  box on;
